'''
Created on Jan 29, 2016

@author: Legendary
'''

import pygame
from pygame.locals import *
from gbls import *
from menu import *
from numpy.oldnumeric.random_array import randint

debug = False
debug = True

global level_designs

level_designs = {   0: [ ("P",(100-50,450),pygame.K_z,True,[1],1),     #0
                         ("P",(200-50,380),pygame.K_s,True,[2,3],1),   #1
                         ("B",(250-50,200),pygame.K_d,True,[4],1),     #2
                         ("B",(350-50,380),pygame.K_c,True,[4],1),     #3
                         ("C",(450-50,250),pygame.K_f,True,[5],3),     #4
                         ("B",(550-50,150),pygame.K_g,True,[6],2),     #5
                         ("C",(720-50,80),pygame.K_h,True,[],5),                           
                         ]
                 }

class Director:
    '''Represents the main object of the game.
 
    The Director object keeps the game on, and takes care of updating it,
    drawing it and propagate events.
 
    This object must be used with Scene objects that are defined later.'''
    def __init__(self,sz):
        self.screen = pygame.display.set_mode(sz,0,24)
        pygame.display.set_caption("SleepyTown")
        self.scene = None
        self.prev_scene = None
        self.quit_flag = False
        self.paused = False
        self.clock = pygame.time.Clock()
        # Makes scenes
#         class Position(object):
#     def __init__(self,level,target_pos,key,alive,link):
        self.music_channel = pygame.mixer.Channel(1)
        self.music_channel.set_volume(0.85)
        self.m_song,_ = load_music("music_0.wav")        
        self.m_song2,_ = load_music("music_1.wav")
        #MUSIC
        self.music_channel.play(self.m_song)
        
        s_mainMenu = Scene_MainMenu(self)
        s_debugScreen = Scene_DebugScreen(self)
        s_level_0 = Scene_LevelScreen(self,0,0,25,2)
        s_attackScene = Scene_AttackScene(self)
        s_winScreen = Scene_WinScreen(self)
        s_loseScreen = Scene_LoseScreen(self)
        s_instrScreen = Scene_InstrScreen(self)
        self.scenes = {
                       'mainMenu' : s_mainMenu, 
                       'debugScreen' : s_debugScreen,
                       'level0' : s_level_0,
                       'attackScreen' : s_attackScene,
                       'winScreen' : s_winScreen, 
                       'loseScreen': s_loseScreen,
                       'instrScreen' : s_instrScreen,
                       }

    
    def loop(self):
        '''The main game loop'''
        while not self.quit_flag:
            self.clock.tick(30)
 
            # Exit events
            events = pygame.event.get()
            for event in events:
                if event.type == pygame.QUIT:
                    self.quit()
                if event.type == pygame.KEYDOWN:
                    if event.key == pygame.K_ESCAPE:
                        self.music_channel.fadeout(2000)
                        self.quit()
                        break
                    
                    if event.key == pygame.K_p: # Handle pausing
                        self.pauseState(self.paused)
 
            # Detect events
            if not self.paused: # I am not paused
                self.scene.on_event(events)
     
                # Update scene
                self.scene.on_update()
     
                # Draw the screen
                self.scene.on_draw()
                
            else: # I am paused
                #Draw PAUSED on screen

                f_Arial_100 = pygame.font.SysFont("Arial", 100)
                pauseText = f_Arial_100.render("GAME PAUSED",True,(255,0,0))
                pauseTextpos = pauseText.get_rect()
                pauseTextpos.center = self.screen.get_rect().center
                self.screen.blit(pauseText, pauseTextpos)
            
            pygame.display.flip()
            
    def pauseState (self,yn):
        if not yn: #Is not paused, set to paused
            self.paused = True
            self.music_channel.pause()
        elif yn:  #Is paused, set to not paused
            self.paused = False
            self.music_channel.unpause()
                
            
    def change_scene(self, scenename):
        "Changes the current scene."
        self.prev_scene = self.scene
        self.scene = self.scenes[scenename]
        if self.scene == self.scenes['mainMenu'] and self.prev_scene != None : #Reset play level if you goto mainmenu
            self.scenes['level0'].reset = True 
    def restore_scene(self):
        tmp = self.scene
        self.scene = self.prev_scene
        self.prev_scene = tmp
        
    def quit(self):
        self.quit_flag = True
        
class Scene:
    """Represents a scene of the game.
 
     Scenes must be created inheriting this class attributes
     in order to be used afterwards as menus, introduction screens,
     etc."""
 
    def __init__(self, director):
        self.director = director
        self.reset = False
 
    def on_update(self):
        "Called from the director and defined on the subclass."
        raise NotImplementedError("on_update abstract method must be defined in subclass.")
 
    def on_event(self, event):
        "Called when a specific event is detected in the loop."
        raise NotImplementedError("on_event abstract method must be defined in subclass.")
 
    def on_draw(self, screen):
        "Called to draw the scene."
        raise NotImplementedError("on_draw abstract method must be defined in subclass.")
    
class Scene_MainMenu(Scene):
    """ Welcome screen of the game, the first one to be loaded.""" 
    def __init__(self, director):
        Scene.__init__(self, director)
        self.director = director
        f_Arial_100 = pygame.font.Font("Font/homespun.ttf", 100)
        f_Arial_30 = pygame.font.SysFont("Arial", 30)
        f_Tahoma_12 = pygame.font.Font("Font/BleedingPixels.ttf", 25)
        self.background = pygame.Surface(director.screen.get_size())
        self.background = self.background.convert()
        self.background.fill(colours['background'])
        
        f_Arial_100 = pygame.font.Font("Font/homespun.ttf", 100)   
        logo_text = f_Arial_100.render("SleepyTown",True,(0,0,0))
        logo_textpos = logo_text.get_rect()
        logo_textpos.center = self.background.get_rect().centerx, self.background.get_rect().top+120
        self.background.blit(logo_text, logo_textpos)
        sublogo_text = f_Tahoma_12.render("... where unslumbering shadows lurk",True,(0,0,0))
        sublogo_textpos = sublogo_text.get_rect()
        sublogo_textpos.topright = plus2Tuple(logo_textpos.bottomright,(0,10))
        self.background.blit(sublogo_text, sublogo_textpos)
        
        #Blit everything to screen
        self.background.blit(self.background,ox)
        
        #Main menu
        self.menu = Menu()#necessary
        self.menu.set_colors( text = colours['menuFont'] , selection = colours['menuSelection'] , background = colours['background'] )
        #menu.set_fontsize(64)#optional
        #menu.set_font('data/couree.fon')#optional
        #menu.move_menu(100, 99)#optional
        self.menu.init(['Play Level 1','Play Level 2','Quit'], self.background)#necessary
        #menu.move_menu(0, 0)#optional
        self.menu.draw()#necessary
    
    def on_update(self):       
        pygame.display.update()
 
    def on_event(self,events):
        for event in events:
            if event.type == KEYDOWN:
                if event.key == K_UP:
                    self.menu.draw(-1) #here is the Menu class function
                if event.key == K_DOWN:
                    self.menu.draw(1) #here is the Menu class function
                if event.key == K_RETURN:
                    if self.menu.get_position() == 0:
                        self.director.change_scene('instrScreen')
                        self.director.music_channel.fadeout(4000)
                        self.director.music_channel.queue(self.director.m_song2)
                    if self.menu.get_position() == 1:
                        self.director.change_scene('debugScreen')
                        self.director.music_channel.fadeout(4000)
                    if self.menu.get_position() == 2:#here is the Menu class function
                        pygame.display.quit()
                        sys.exit()               
 
    def on_draw(self):
        self.director.screen.blit(self.background,ox)
        
class Scene_DebugScreen(Scene):
    """ Pink screen of death, for debuggin 'n stuff :> """ 
    def __init__(self, director):
        Scene.__init__(self, director)
        self.director = director
        self.background = pygame.Surface(director.screen.get_size())
        self.background = self.background.convert()
        self.background.fill(colours['debugPink'])
        bsod, _ = load_img("Windows_9X_BSOD.png")
        bsod = pygame.transform.scale(bsod,self.director.screen.get_size())
        self.background.blit(bsod,ox)
        #Blit everything to screen
        self.background.blit(self.background,ox)
            
    def on_update(self):       
        pygame.display.update()
 
    def on_event(self,events):
        for event in events:
            if event.type == KEYDOWN:
                    if event.key == K_q:
                        self.director.restore_scene() #here is the Menu class function
    
    def on_draw(self):
        self.director.screen.blit(self.background,ox)

class Scene_InstrScreen(Scene):
    """ Instructions """ 
    def __init__(self, director):
        Scene.__init__(self, director)
        self.director = director
        self.background = pygame.Surface(director.screen.get_size())
        self.background = self.background.convert()
        self.background.fill(colours['background'])
        
        #Blit everything to screen
        self.background.blit(self.background,ox)
            
    def on_update(self):       
        pygame.display.update()
 
    def on_event(self,events):
        for event in events:
            if event.type == KEYDOWN:
                    self.director.change_scene('level0') #here is the Menu class function
    
    def on_draw(self):
        f_Arial_100 = pygame.font.Font("Font/homespun.ttf", 100)   
        logo_text = f_Arial_100.render("Instructions",True,(0,0,0))
        logo_textpos = logo_text.get_rect()
        logo_textpos.center = (400,100)
        self.background.blit(logo_text, logo_textpos)
        
        f_Arial_1200 = pygame.font.Font("Font/homespun.ttf", 30)
        t1 = f_Arial_1200.render("Walk into available circles of power",True,(0,0,0))
        t1pos = t1.get_rect()
        t1pos.center = (400,200)
        self.background.blit(t1, t1pos)
        t1 = f_Arial_1200.render("by pressing keyboard keys.",True,(0,0,0))
        t1pos = t1.get_rect()
        t1pos.center = (400,250)
        self.background.blit(t1, t1pos)
        t1 = f_Arial_1200.render("Break magical barriers to progress.",True,(0,0,0))
        t1pos = t1.get_rect()
        t1pos.center = (400,300)
        self.background.blit(t1, t1pos)
        t1 = f_Arial_1200.render("Press corresponding keys when next to a barrier.",True,(0,0,0))
        t1pos = t1.get_rect()
        t1pos.center = (400,350)
        self.background.blit(t1, t1pos)
        t1 = f_Arial_1200.render("Attack adjacent cultist by pressing shaking keys.",True,(0,0,0))
        t1pos = t1.get_rect()
        t1pos.center = (400,400)
        self.background.blit(t1, t1pos)
        t1 = f_Arial_1200.render("Kill all the cultists to stop the ritual.",True,(0,0,0))
        t1pos = t1.get_rect()
        t1pos.center = (400,450)
        self.background.blit(t1, t1pos)
        
        
        f_Arial = pygame.font.SysFont("Arial", 30)   
        l = f_Arial.render("Press any key to continue...",True,(0,0,0))
        l_textpos = l.get_rect()
        l_textpos.center = (400,500)
        self.background.blit(l, l_textpos)
        self.director.screen.blit(self.background,ox)


class Scene_WinScreen(Scene):
    """ a win.rar is you """ 
    def __init__(self, director):
        Scene.__init__(self, director)
        self.director = director
        self.background = pygame.Surface(director.screen.get_size())
        self.background = self.background.convert()
        self.background.fill(colours['background'])
        
        #Blit everything to screen
        self.background.blit(self.background,ox)
            
    def on_update(self):       
        pygame.display.update()
 
    def on_event(self,events):
        for event in events:
            if event.type == KEYDOWN:
                    self.director.change_scene('mainMenu') #here is the Menu class function
    
    def on_draw(self):
        f_Arial_100 = pygame.font.Font("Font/homespun.ttf", 100)   
        logo_text = f_Arial_100.render("Level Clear!",True,(0,0,0))
        logo_textpos = logo_text.get_rect()
        logo_textpos.center = (400,300)
        self.background.blit(logo_text, logo_textpos)
        f_Arial = pygame.font.SysFont("Arial", 30)   
        l = f_Arial.render("Press any key to continue...",True,(0,0,0))
        l_textpos = l.get_rect()
        l_textpos.midtop = logo_textpos.midbottom
        self.background.blit(l, l_textpos)
        self.director.screen.blit(self.background,ox)
        
class Scene_LoseScreen(Scene):
    """ u sucked :( """ 
    def __init__(self, director):
        Scene.__init__(self, director)
        self.director = director
        self.background = pygame.Surface(director.screen.get_size())
        self.background = self.background.convert()
        self.background.fill(colours['background'])
        
        #Blit everything to screen
        self.background.blit(self.background,ox)
            
    def on_update(self):       
        pygame.display.update()
 
    def on_event(self,events):
        for event in events:
            if event.type == KEYDOWN:
                    self.director.change_scene('mainMenu') #here is the Menu class function
    
    def on_draw(self):
        f_Arial_100 = pygame.font.Font("Font/BleedingPixels.ttf", 100)   
        logo_text = f_Arial_100.render("You Lost!",True,(155,0,0))
        logo_textpos = logo_text.get_rect()
        logo_textpos.center = (400,300)
        self.background.blit(logo_text, logo_textpos)
        f_Arial = pygame.font.SysFont("Arial", 30)   
        l = f_Arial.render("Press any key to continue...",True,(0,0,0))
        l_textpos = l.get_rect()
        l_textpos.midtop = logo_textpos.midbottom
        self.background.blit(l, l_textpos)
        self.director.screen.blit(self.background,ox)
        
class Scene_LevelScreen(Scene):
    """ Level Screen, featuring game play """ 
    def __init__(self, director,levelno,designNo,moves,nocult):
        Scene.__init__(self, director)
        self.levelno = levelno
        self.director = director
        self.state = 0     
        self.designNo = designNo  
        self.maxc = nocult
        self.level = Level(level_designs[self.designNo],self.maxc) 
        self.currAsset = None
        self.currMoves = moves
        self.maxMoves = moves
        self.localfont = pygame.font.SysFont('Arial',20)
        self.localfontSymbs = pygame.font.Font('Font/IconBitOne.ttf',20)

    
    def injectSomething(self,x):
        if x == "failed":
            pass
        if x == "success":
            self.currAsset.attackSuccess()
        self.currAsset = None
                         
            
    def on_update(self): 
        pygame.display.update()
        if self.reset:
            print "Reset!"
            self.reset = False
            self.level = Level(level_designs[self.designNo],self.maxc)
            self.currMoves = self.maxMoves
            self.currAsset = None
        if self.currMoves == 0: #lose level!
            self.director.change_scene("loseScreen") #here is the Menu class function
        if self.level.cults == 0: #WIN level!
            self.director.change_scene("winScreen") #here is the Menu class function
 
    def on_event(self,events):
        for event in events:
            if event.type == KEYDOWN:
                self.currMoves -=1
                if event.key == K_q:
                    self.director.change_scene("mainMenu") #here is the Menu class function
                if event.key == K_o:
                    print self.level #here is the Menu class function
                if event.key == K_COMMA:
                    self.director.change_scene("debugScreen") #here is the Menu class function
                for t,k,a in self.level.get_keys():
                    if event.key == k and t == "Move":
                        self.level.move_playerToAsset(a[0])
                    if event.key == k and t == "KillBlock":
                        self.level.kill_block(a[0], a[1])
                    if event.key == k and t == "HurtCultist":
                        x = self.level.attack_cultist(a[0])
                        self.currAsset = a[0]
                        self.director.change_scene('attackScreen')
                        self.director.scene.insertTargets(x)

    def on_draw(self):
        background = pygame.Surface(self.director.screen.get_size())
        background = background.convert()
        background.fill(colours['background'])
        self.level.draw_alive(background) 
        self.level.draw_availableKeys(background)
        text2 = "Moves remaining:"
        output_text2 = self.localfont.render(text2,True,(0,0,0))
        output_textpos2 = output_text2.get_rect()
        output_textpos2.topleft = (10,10)
        background.blit(output_text2, output_textpos2)   
        text3 = "t"*self.currMoves
        output_text3 = self.localfontSymbs.render(text3,True,(0,0,0))
        output_textpos3 = output_text3.get_rect()
        output_textpos3.topleft = (150,10)      
        background.blit(output_text3, output_textpos3)      
        self.director.screen.blit(background,ox)
        
class Scene_AttackScene(Scene):
    """ Mini games to break blocks """ 
    def __init__(self, director):
        Scene.__init__(self, director)
        self.director = director
        self.background = pygame.Surface(director.screen.get_size())
        self.background = self.background.convert()
        self.background.fill(colours['debugPink'])
        #Blit everything to screen
        self.background.blit(self.background,ox)
        self.currentCount = 0
        self.maxCount = 0
        self.currentKey = "" 
        self.targets = []
        self.time = 10.00 #time in seconds
        pygame.time.set_timer(USEREVENT+1, 100)#1 second is 1000 milliseconds
        self.localfont = pygame.font.SysFont('Arial',30)
        self.fightingsounds = []
        for i in range(6): self.fightingsounds.append(load_sound("punch_%d.wav"%(i)))
        
    def insertTargets(self,ls):
        self.__init__(self.director)
        self.targets = ls
        self.currentCount = 0
        self.maxCount = len(self.targets) - 1
             
    def on_update(self):       
        #Show the timer
        if self.currentCount > self.maxCount:
            self.director.restore_scene()
            self.director.scene.injectSomething("success")
        else:
            self.currentKey = self.targets[self.currentCount] 
#             print self.currentKey, chr(self.currentKey)
            
        
  
    def on_event(self,events):
        for event in events:
            if event.type == KEYDOWN:
                if event.key == self.currentKey:
                    s, _ = self.fightingsounds[randint(0, len(self.fightingsounds))]
                    s.play()
                    self.currentCount += 1
                elif event.key == K_p:
                    pass
                else:
                    self.director.restore_scene()
                    self.director.scene.injectSomething("failed")

            elif event.type == USEREVENT+1:
                self.time -= 0.1
                if self.time < 0.00:
                    self.director.restore_scene()
                    self.director.scene.injectSomething("failed")
                
                             
    def on_draw(self):
        scale = max (float(self.currentCount) / float(self.maxCount),0.0)
        redness = (204.0*scale,0.0,0.0)
        self.background.fill(colours['debugPink'])
        text0 = "%s"%(chr(self.currentKey))
        output_text0 = pygame.font.Font('Font/Where My Keys.otf',randint(300,400)).render(text0,True,redness)
        output_textpos0 = output_text0.get_rect()
        spread = (-5,6)
        output_textpos0.center = plus2Tuple( (400,300),(randint(spread[0],spread[1]),randint(spread[0],spread[1])) )
        self.background.blit(output_text0, output_textpos0)         
        
        text1 = "%.2f"% (max(self.time,0.0))
        output_text1 = self.localfont.render(text1,True,(0,0,0))
        output_textpos1 = output_text1.get_rect()
        output_textpos1.bottomright = (800-10,600-10)
        self.background.blit(output_text1, output_textpos1)
           
        text2 = "Time remaining:"
        output_text2 = self.localfont.render(text2,True,(0,0,0))
        output_textpos2 = output_text2.get_rect()
        output_textpos2.bottomright = (800-80,600-10)
        self.background.blit(output_text2, output_textpos2) 
        self.director.screen.blit(self.background,ox)
      
        
        
        
        